#include <stdio.h>
#include <stdlib.h>
#include <osbind.h>
#include <string.h>
#include <ctype.h>

#ifdef __PUREC__
 #include <ext.h>
 /* #define FA_RDONLY FA_READONLY */
#endif

#include "elib.h"

 
/*
 * delete_file
 *
 * given the name of a file, attempt to delete it and return zero on success,
 * non-zero if not.  'force' parameter, if non-zero, will try to change the
 * mode of a file from read-only to delete it.
 */
int delete_file(char *file, char force)

{
	if (force) 
	{
		int mode;

		if ((mode = Fattrib(file, 0, 0)) & FA_RDONLY)
			Fattrib(file, 1, mode&(!FA_RDONLY));
	}

	return Fdelete(file);
}

/*
 * cmptime
 *
 * given two files, return positive if the first has a more recent modification
 * date/time, zero if the files have the same modification date/time or
 * negative if the second is more recent.
 */
long cmptime(const struct _dta *a, const struct _dta *b)
{
	return ((((unsigned long)a->dta_date) << 16) | (unsigned long)a->dta_time) -
		   ((((unsigned long)b->dta_date) << 16) | (unsigned long)b->dta_time);
}


#define ERR(s, c)	if(opterr){\
	char errbuf[3];\
	errbuf[0] = c; errbuf[1] = '\r'; errbuf[2] = '\n';\
	(void) write(2, argv[0], (unsigned)strlen(argv[0]));\
	(void) write(2, s, (unsigned)strlen(s));\
	(void) write(2, errbuf, 3);}

extern int strcmp();
extern char *strchr();

int	opterr = 1;
int	Optind = 1;
int	optopt;
char	*optarg;

int getopt(argc, argv, opts)
int	argc;
char	**argv, *opts;
{
	static int sp = 1;
	register int c;
	register char *cp;

	if(sp == 1)
		if(Optind >= argc ||
		   argv[Optind][0] != '-' || argv[Optind][1] == '\0')
			return(EOF);
		else if(!strcmp(argv[Optind], "--")) {
			Optind++;
			return(EOF);
		}
	optopt = c = argv[Optind][sp];
	if(c == ':' || (cp=strchr(opts, c)) == NULL) {
		ERR(": illegal option -- ", c);
		if(argv[Optind][++sp] == '\0') {
			Optind++;
			sp = 1;
		}
		return('\0');
	}
	if(*++cp == ':') {
		if(argv[Optind][sp+1] != '\0')
			optarg = &argv[Optind++][sp+1];
		else if(++Optind >= argc) {
			ERR(": option requires an argument -- ", c);
			sp = 1;
			return('\0');
		} else
			optarg = argv[Optind++];
		sp = 1;
	} else {
		if(argv[Optind][++sp] == '\0') {
			sp = 1;
			Optind++;
		}
		optarg = NULL;
	}
	return(c);
}


/*
 * mark_list
 *
 * given a list of numbers (separated by commas with ranges specified by -'s),
 * fill in the given character array with 0's for numbers not specified and
 * 1's for numbers specified.  The size of the character array is also given.
 * Return 0 on success, non-zero means failure, such as an out-of-range number
 * in the given list.
 */
int mark_list(char *num_list, char *mark_list, int max_num)

{
	char *cur_pos, *new_pos;
	int i, j;

	for (i=0; i<max_num; i++)
		mark_list[i] = '\0';

	for (cur_pos = num_list; *cur_pos != '\0'; cur_pos = ++new_pos) {
		if (((i = (int) strtol(cur_pos, &new_pos, 10)) < 1) || (i > max_num) ||
			(new_pos == NULL))
			return 1;
		
		if (*(new_pos = skip_whitespace(new_pos)) == '-') {
			cur_pos = new_pos+1;
			if (((j = (int) strtol(cur_pos, &new_pos, 10)) < i) ||
				(j > max_num) || (new_pos == NULL))
				return 1;
			for (i--, j--; i <= j; i++)
				mark_list[i] = '\1';
		}
		else if (*new_pos == ',')
			mark_list[i-1] = '\1';
		else if (*new_pos ==  '\0') {
			mark_list[i-1] = '\1';
			return 0;
		}
		else if (isdigit(*new_pos)) {
			mark_list[i-1] = '\1';
			new_pos--;
		}
	}

	return 0;
}

/*
 * hashpjw
 *
 * hash function.  Used in P. J. Weinberger's portable C compiler and
 * gotten from page 436 of the Dragon book on compilers (Compilers: Principles,
 * Techniques and Tools, by Aho, Sethi and Ullman.  Copyright (c) 1986 by
 * Bell Telephone Laboratories).
 */
int hashpjw(char *s)

{
	char *p;
	unsigned long h = 0, g;

	for (p = s; *p != EOS; p++) 
	{
		h = (h << 4) + (*p);
		g = h & 0xf0000000l;
		if (g) 
		{
			h ^= (g >> 24);
			h ^= g;
		}
	}

	return (int)(h % HASH_TAB_SIZE);
}


/*
 * skip_whitespace
 *
 * given a string pointer, skip the whitespace and return the pointer to
 * the new spot.
 */
char *skip_whitespace(char *str)

{
	while (isspace(*str))
		str++;

	return str;
}



/*
 * zap_trailing_nl
 *
 * on strings which have been read in using fgets(), take out the trailing
 * newline.  If no newline, read the input stream until one is encountered.
 */
void zap_trailing_nl(char *str, int max_ch, FILE *stream)

{
	int i = (int)strlen(str), ch;

	if ((i > 0) && (str[i-1] == '\n'))
		str[i-1] = '\0';
	else if (i >= max_ch)
		while (((ch = getc(stream)) != EOF) && (ch != '\n'));
}

/*
 * is_special
 *
 * given a directory name, return non-zero if it is 'special' (i.e. '.' or
 * '..') or zero if it is normal
 */
int is_special(char *d_name)

{
	int d_len;

	if ((d_name == NULL) || ((d_len = (int)strlen(d_name)) > 2))
		return 0;
	else if (d_len == 2)
		return (!strcmp(d_name, ".."));
	else if (d_len == 1)
		return (d_name[0] == '.');
	else	/* null directory name */
		return 0;
}



/*
 * append_dir_to_path
 *
 * given a path and a directory name, return a pointer to memory with
 * the directory appended to the path
 */

char *append_dir_to_path(char *path, char *dir)
{
	static char *dir_sep = PATH_SEPARATOR;
	char *ret_val, sep_byte;
	int path_len, mem_nec;

	mem_nec = (path_len = (int)strlen(path)) +
			  (sep_byte = ((path[path_len-1] != dir_sep[0]) ? 1 : 0)) +
			  (int)strlen(dir) + 1;
	if ((ret_val = malloc((size_t) mem_nec)) == NULL) 
	{
		printf("append_dir_to_path: memory allocation failure!\n");
		exit(1);
	}
	else 
	{
		strcpy(ret_val, path);
		if (sep_byte)
			strcat(ret_val, dir_sep);
		strcat(ret_val, dir);

		return(ret_val);
	}
	return(0);
}



/*
 *	format_dir()
 *
 *	Input:
 *		dir - the directory as specified by user
 *		append_slash - non-zero if a slash should be appended, zero otherwise
 *	Output:
 *		formatted_dir - a formatted version of 'dir'
 */

void format_dir(char *dir, char app_slash, char *formatted_dir)
{
	char cur_dir[FILENAME_MAX];
	int i;

	getcwd(cur_dir, FILENAME_MAX);
	chdir(dir);
	getcwd(formatted_dir, FILENAME_MAX);
	for (i=0; formatted_dir[i] != EOS; i++)
		if (formatted_dir[i] == '/')
			formatted_dir[i] = '\\';
	if ((app_slash) && (formatted_dir[i - 1] != '\\'))
		strcat(formatted_dir, "\\");
	chdir(cur_dir);
}


/*
 *	ask_user()
 *
 *	Input:
 *	Output:
 *	Comments:
 */

int ask_user(char *buf)
{
	int done = 0;
	char ans[10];
	int retval;

	while(!done)
	{
		fprintf(stdout, "%s", buf);
		if (gets(ans) != NULL)
		{
			switch(*ans)
			{
				case 'n':
				case 'N':
					retval = 0;
					done = 1;
					break;
				case 'y':
				case 'Y':
					retval = 1;
					done = 1;
					break;
				case 'q':
				case 'Q':
					exit(0);
				default:
					break;
			}
		}
		else
			exit(1);
	}
	return(retval);
}
